//For arraylist
import java.util.ArrayList;
//Required to use HashMap
import java.util.HashMap;
//for tokenizing string
import java.util.StringTokenizer;

/**
 * Builds a hashmap of words from lines of texts and provide some utility methods.
 * 
 * @author Abu Shahriar
 * @version 2012.10.01
 */
public class WordFrequency 
{
    private String delimiters;//delimiter string
    private HashMap<String, Integer> wordTable;//the hashmap for words
    /**
     * Constructor for objects of class WordFrequency
     */
    public WordFrequency()
    {
        delimiters  = " ,;.!?:"; //set the delimiters
        wordTable = new HashMap<String, Integer>();//create an empty hashmap initially
    }

    /**
     * Builds a hashmap of words from lines of texts.
     * 
     * @param  lines lines of text 
     */
    public void buildWordFrequencyList(ArrayList<String> lines)
    {
        for(String line:lines)
        {
                StringTokenizer st = new StringTokenizer(line,delimiters);//break a line into words using the delimiters
                while (st.hasMoreTokens()) //continue as long as there is a word to be processed
                {
                    //System.out.println(st.nextToken());
                    String word = st.nextToken();//get the next word
                   // word = word.toUpperCase();
                    Integer count = wordTable.get(word);//get the frequency of the word
                    if(count != null)
                        //the word is already in the table, increment its count
                        wordTable.put(word,count+1);
                    else//new word, set the counter to 1
                        wordTable.put(word,1);
                }
            }
        }
    
    /**
     * Find the frequency of a paritcular word in the table.
     * 
     * @param  word the word for which the frequency to be searched for
     * @return the frequency of the word
     */
    public int findWordFrequency(String word)
    {
        //word = word.toUpperCase();
        if(wordTable.isEmpty() == false){// the hashmap is not empty
            Integer frequency = wordTable.get(word);
            if(frequency != null)// the word is present in the hashmap
                return frequency;
        }
        return 0;// return 0 when the hashmap is empty or the word is not in the hashmap
    }
    
    /**
     * Finds a list of words having a given frequency or more.
     * 
     * @param frequency the minimum frequency the words must have
     * @return a list of words
     */
    public ArrayList<String> findWords(int frequency)
    {
        ArrayList<String> words = new ArrayList<String>();//create an empty word list
        for (String s : wordTable.keySet()) { //go through the entire hashmap
                //if the word's frequency is equal to or larger than the given frequency
                //add the work to the list
               if(wordTable.get(s) >= frequency)
               {
                   words.add(s);
               }
        }        
        return words;
    }
    /**
     * Find the maximum frequency in the list
     * 
     * @return the maximum frequency
     */
    public int findMaximumFrequency(){
        int max = 0; //start with 0 as maximum, as freuqnecy of word cannot be negative
        for (String s : wordTable.keySet()) { //go through the entire hashmap
               if(wordTable.get(s) > max) //if a new maximum is found
               {
                   max = wordTable.get(s); //update the maximum
               }
        }        
        return max;
    }

    /**
     * Return a list of words having maximum frequency.
     * 
     * @return a list of words
     */
    public ArrayList<String> findMaximumFrequencyWords()
    {
        ArrayList<String> words;//create an empty word list
        int max = findMaximumFrequency();//find the maximum frequency
        words = findWords(max);//get the list of word having maximum frequency or more
        return words;
    }
    
}